//
//  UndoWindow.m
//  Undo Inspector
//
//  Created by Patrick Geiller on 19/06/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "UndoWindow.h"
#import "DummyView.h"


@implementation UndoWindow


- (void)awakeFromNib
{
	NSFont* font = [NSFont fontWithName:@"Andale Mono" size:9];
	[undoStackView setFont:font];
	[redoStackView setFont:font];
	[callLogView setFont:font];
	
	objects = [[NSMutableDictionary alloc] init];
}

- (void)delayDumpUndo
{
	[self performSelector:@selector(dumpUndo) withObject:nil afterDelay:0];
}

//
// Dump undo and redo stacks by fetching NSUndoManager's stack instance variables
//
- (void)dumpUndo
{
	@try
	{
		NSUndoManager* undoManager = [self undoManager];
		id v1, v2;
		object_getInstanceVariable(undoManager, "_undoStack", &v1);
		object_getInstanceVariable(undoManager, "_redoStack", &v2);
		[undoStackView setString:[NSString stringWithFormat:@"(%d entries) %@", [v1 count], [v1 description]] ];
		[redoStackView setString:[NSString stringWithFormat:@"(%d entries) %@", [v2 count], [v2 description]] ];
	}
	@catch (NSException* e)
	{
	}
}

//
// Dump lives objects and their retain count
//	Called by DummyObject's retain and release
//
- (void)updateObjectsWithObject:(id)o andRetainCount:(int)count
{
	[objects setValue:[NSNumber numberWithInt:count] forKey:[NSString stringWithFormat:@"%x", o]];

	NSString* str = [NSString string];
	NSString* str2 = nil;
	id keys = [objects allKeys];
	int liveObjects = 0;
	for (id key in keys)
	{
		int retainCount = [[objects valueForKey:key] intValue];
		if (retainCount == 0)	continue;
		if (key == o)	str2 = @"LLLLL";
		else			str2 = [NSString stringWithFormat:@"%x=%d %@", key, retainCount, str];
		liveObjects++;
		str = str2;
	}
	str = [NSString stringWithFormat:@"(%d live objects) %@", liveObjects, str];
	[callLogView setString:str];
}

//
// Clear undo stack
//
- (IBAction)clearUndoStack:(id)sender
{
	[[self undoManager] removeAllActions];
	[self delayDumpUndo];
}


@end
